@@ -94,10 +94,6 @@ sub processfile {
 		}
 	}
 
-	if (exists $fields{"packages-list"}) {
-		$fields{packages}="task-fields";
-	}
-	
 	foreach (qw{task section relevance description key depends packages}, 
 	         grep(/^test-(.*)/, keys %fields)) {
 		print OUT ucfirst($_).": ".$fields{$_}."\n" if length $fields{$_};
