@@ -60,7 +60,7 @@ sub read_task_desc {
 				}
 			}
 			else {
-				print STDERR "parse error in stanza $. of $desc\n";
+				warning "parse error in stanza $. of $desc";
 			}
 		}
 		if (%data) {
@@ -124,7 +124,7 @@ sub package_avail {
 		}
 	}
 
-	return $avail_pkgs{$package};
+	return $avail_pkgs{$package} || package_installed($package);
 }
 
 my %installed_pkgs;
@@ -184,9 +184,16 @@ sub task_packages {
 	my $task=shift;
 	my $aptitude_tasks=shift;
 	
-	my @list;
+	my %list;
+
+	# key packages are always included
+	if (ref $task->{key}) {
+		map { $list{$_}=1 } @{$task->{key}};
+	}
+		
 	if ($task->{packages} eq 'task-fields') {
-		# task-fields method is built-in for speed.
+		# task-fields method is built-in for speed and to support
+		# aptitude task definitions
 		if ($aptitude_tasks) {
 			return '~t^'.$task->{task}.'$';
 		}
@@ -197,7 +204,7 @@ sub task_packages {
 				if (/^Task: (.*)/m) {
 					my @tasks=split(", ", $1);
 					if (grep { $_ eq $task->{task} } @tasks) { 
-						push @list, $1 if /^Package: (.*)/m;
+						$list{$1}=1 if /^Package: (.*)/m;
 					}
 				}
 			}
@@ -209,11 +216,27 @@ sub task_packages {
 		# implemented externally.
 		return "~pstandard", "~prequired", "~pimportant";
 	}
+	elsif ($task->{packages} eq 'manual') {
+		# manual package selection is a special case
+		return;
+	}
 	else {
 		# external method
-		@list=grep { package_avail($_) } split("\n", `$packagesdir/$task->{packages} $task->{task}`);
+		my ($method, @params);
+		if (ref $task->{packages}) {
+			@params=@{$task->{packages}};
+			$method=shift @params;
+		}
+		else {
+			$method=$task->{packages};
+		}
+		
+		map { $list{$_}=1 }
+			grep { package_avail($_) }
+			split(' ', `$packagesdir/$method $task->{task} @params`);
 	}
-	return @list;
+
+	return keys %list;
 }
 
 # Given a task hash, runs any test program specified in its data, and sets
@@ -305,7 +328,7 @@ sub usage {
 	print STDERR gettext(q{Usage:
 tasksel install <task>
 tasksel remove <task>
-tasksel [options]; where options is any combination of:
+tasksel [options]
 	-t, --test          test mode; don't really do anything
 	    --new-install   automatically install some tasks
 	    --list-tasks    list tasks that would be displayed and exit
@@ -319,7 +342,8 @@ sub getopts {
 	my %ret;
 	Getopt::Long::Configure ("bundling");
 	if (! GetOptions(\%ret, "test|t", "new-install", "list-tasks",
-		   "task-packages=s@", "task-desc=s")) {
+		   "task-packages=s@", "task-desc=s",
+		   "debconf-apt-progress=s")) {
 		usage();
 		exit(1);
 	}
@@ -378,8 +402,6 @@ sub main {
 		exit(0);
 	}
 	
-	# Now work out what to tell aptitude to install.
-	my @aptitude_install;
 	if (! $options{"new-install"}) {
 		# Don't install hidden tasks if this is not a new install.
 		map { $_->{_install} = 0 } grep { $_->{_display} == 0 } @tasks;
@@ -396,7 +418,6 @@ sub main {
 	
 	# The interactive bit.
 	my $interactive=0;
-	my $manual_selection=0;
 	my @list = order_for_display(grep { $_->{_display} == 1 } @tasks);
 	if (@list && ! $options{install} && ! $options{remove}) {
 		$interactive=1;
@@ -419,22 +440,25 @@ sub main {
 		my @default = grep { $_->{_display} == 1 && ($_->{_install} == 1 || $_->{_installed} == 1) } @tasks;
 		my $tmpfile=`tempfile`;
 		chomp $tmpfile;
-		system($debconf_helper, $tmpfile,
+		my $ret=system($debconf_helper, $tmpfile,
 			task_to_debconf("shortdesc", @list),
 			task_to_debconf("shortdesctrans", @list),
 			task_to_debconf("shortdesc", @default),
-			$question);
+			$question) >> 8;
+		if ($ret == 30) {
+			exit 10; # back up
+		}
+		elsif ($ret != 0) {
+			error "debconf failed to run";
+		}
 		open(IN, "<$tmpfile");
-		my $ret=<IN>;
+		$ret=<IN>;
 		if (! defined $ret) {
 			die "tasksel canceled\n";
 		}
 		chomp $ret;
 		close IN;
 		unlink $tmpfile;
-		if ($ret=~/manual package selection/) {
-			$manual_selection=1;
-		}
 		
 		# Set _install flags based on user selection.
 		map { $_->{_install} = 0 } @list;
@@ -463,22 +487,41 @@ sub main {
 		}
 	}
 
-	# Add tasks to install.
-	my @to_install=map { task_packages($_, 1) } grep { $_->{_install} } @tasks;
-	push @aptitude_install, @to_install;
+	# Add tasks to install and see if any selected task requires manual
+	# selection.
+	my @aptitude_install;
+	my $manual_selection=0;
+	foreach my $task (grep { $_->{_install} } @tasks) {
+		push @aptitude_install, task_packages($task, 1);
+		if ($task->{packages} eq 'manual') {
+			$manual_selection=1;
+		}
+	}
 	
-	# Clear screen before running aptitude.
-	if ($interactive && (@aptitude_remove || @aptitude_install) && ! $options{test}) {
-		system("clear");
+	my $aptitude;
+	if ($manual_selection) {
+		# Manaul selection and task installs, as best
+		# aptitude can do it currently. Disables use of
+		# debconf-apt-progress.
+		$aptitude="aptitude";
+	}
+	elsif (-x "/usr/bin/debconf-apt-progress") {
+		$aptitude="debconf-apt-progress";
+		$aptitude.=" $options{'debconf-apt-progress'}"
+			if exists $options{'debconf-apt-progress'};
+		$aptitude.=" -- aptitude -q";
+	}
+	else {
+		$aptitude="aptitude";
 	}
 
 	# Remove any packages we were asked to.
 	if (@aptitude_remove) {
 		if ($options{test}) {
-			print "aptitude remove ".join(" ", @aptitude_remove)."\n";
+			print "$aptitude -y remove ".join(" ", @aptitude_remove)."\n";
 		}
 		else {
-			my $ret=system("aptitude", "remove", @aptitude_remove) >> 8;
+			my $ret=system(split(' ', $aptitude), "-y", "remove", @aptitude_remove) >> 8;
 			if ($ret != 0) {
 				error gettext("aptitude failed")." ($ret)";
 			}
@@ -489,7 +532,7 @@ sub main {
 	if (@aptitude_install || $manual_selection) {
 		# If the user selected no tasks and manual package
 		# selection, run aptitude w/o the --visual-preview parameter.
-		if (! @to_install && $manual_selection) {
+		if (! @aptitude_install && $manual_selection) {
 			if ($options{test}) {
 				print "aptitude\n";
 			}
@@ -501,17 +544,14 @@ sub main {
 			}
 		}
 		else {
-			# Manaul selection and task installs, as best
-			# aptitude can do it currently.
 			if ($manual_selection) {
 				unshift @aptitude_install, "--visual-preview";
 			}
-			
 			if ($options{test}) {
-				print "aptitude --without-recommends -y install ".join(" ", @aptitude_install)."\n";
+				print "$aptitude --without-recommends -y install ".join(" ", @aptitude_install)."\n";
 			}
 			else {
-				my $ret=system("aptitude", "--without-recommends", "-y", "install", @aptitude_install) >> 8;
+				my $ret=system(split(' ', $aptitude), "--without-recommends", "-y", "install", @aptitude_install) >> 8;
 				if ($ret != 0) {
 					error gettext("aptitude failed");
 				}
